/**
 * CONTROLLER: LoginController
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .controller( 'LoginController', LoginController );

  function LoginController( $filter, $interval, $rootScope, $state, $scope, LoginService, currentCommo, SettingsService ) {
    var vm = this,
      listener;

    vm.credentials = LoginService.getOrCreateClearCredentials();
    vm.commoditySelection = currentCommo.name;
    vm.login = login;
    vm.changeCommo = changeCommo;

    $scope.$on( '$destroy', listener );

    ////////////


    function login() {
      var filterTranslate = $filter( 'translate' );

      LoginService.login( vm.credentials.email, vm.credentials.password )
        .then( loginSuccess )
        .catch( loginError );

      ////////////

      /**
       * Login Success
       * Save login info
       * Start the app
       */
      function loginSuccess() {
        LoginService.saveClearCredentials( vm.credentials );
        $rootScope.$emit( 'startPolling' );
        $state.go( 'session.map', { reload: true } );
      }

      /**
       * Login error
       * Show error message
       * @param error
       * @todo: fix or remove unused countdown implementation
       */
      function loginError( error ) {
        vm.password = '';

        if ( error.data && error.data.remainingTime ) {
          vm.remainingTime = Math.ceil( error.data.remainingTime / 1000 ) + 10;
          $scope.$emit( 'toast:error', filterTranslate( 'APP.START.LOGIN.ERROR_BLOCKED' )( { 'remainingTime': vm.remainingTime } ) );
          startCountdown();
        } else {
          $scope.$emit( 'toast:error', filterTranslate( global.textTranslateHelper.translateLoginError( error.data.code ) ) );
        }

        ////////////

        function startCountdown() {
          var countdownPromise = $interval( function() {
            --vm.remainingTime;
            if ( vm.remainingTime <= 0 ) { $interval.cancel( countdownPromise ); }
          }, 1000 );
        }
      }
    }

    /**
     * Change commodity
     */
    function changeCommo() {
      SettingsService.setCommodity( vm.commoditySelection );
    }
  }

} )( this );
