( function() {
  'use strict';


  angular
    .module( 'lng' )
    .controller( 'DashboardController', DashboardController );

  function DashboardController( $ionicLoading, $scope, $q, DashboardService, $interpolate ) {
    var vm = this,

      // Define promises separately to be able to catch errors separately
      fleetOverviewPromise = DashboardService.getFleetOverview()
        .then( assignGaugeConfig )
        .catch( errorGaugeConfig ),

      tendersPromise = DashboardService.getLatestTenders()
        .then( assignTenders )
        .catch( errorTenders ),

      // Show the dashboard once all promises are resolved
      promises = [ fleetOverviewPromise, tendersPromise ];

    vm.fleetOverviewReady = false;
    vm.requestTenders = false;
    vm.dashboardReady = false;

    $ionicLoading.show( {
      content: '',
      animation: 'fade-in'
    } );

    $q.all( promises )
      .then( dashboardReady )
      .finally( $ionicLoading.hide );

    ////////////

    function assignGaugeConfig( fleetOverview ) {

      var segments = [
        {
          value: _.get( fleetOverview, 'loaded.count' ),
          typeKey: 'APP.DASHBOARD.FLEET_OVERVIEW.LOADED',
          typeWithShipsKey: 'APP.DASHBOARD.FLEET_OVERVIEW.LOADED_SHIPS',
          className: 'lng-badge-vessel-loaded'
        },
        {
          value: _.get( fleetOverview, 'ballast.count' ),
          typeKey: 'APP.DASHBOARD.FLEET_OVERVIEW.BALLAST',
          typeWithShipsKey: 'APP.DASHBOARD.FLEET_OVERVIEW.BALLAST_SHIPS',
          className: 'lng-badge-vessel-ballast'
        },
        {
          value: _.get( fleetOverview, 'open.count' ),
          typeKey: 'APP.DASHBOARD.FLEET_OVERVIEW.OPEN',
          typeWithShipsKey: 'APP.DASHBOARD.FLEET_OVERVIEW.OPEN_SHIPS',
          className: 'lng-badge-vessel-open'
        },
        {
          value: _.get( fleetOverview, 'laidUp.count' ),
          typeKey: 'APP.DASHBOARD.FLEET_OVERVIEW.LAID_UP',
          typeWithShipsKey: 'APP.DASHBOARD.FLEET_OVERVIEW.LAID_UP_SHIPS',
          className: 'lng-badge-vessel-laid-up'
        },
        {
          value: _.get( fleetOverview, 'underConstruction.count' ),
          typeKey: 'APP.DASHBOARD.FLEET_OVERVIEW.UNDER_CONSTRUCTION',
          typeWithShipsKey: 'APP.DASHBOARD.FLEET_OVERVIEW.UNDER_CONSTRUCTION_SHIPS',
          className: 'lng-badge-vessel-underconstruction'
        },
        {
          value: _.get( fleetOverview, 'inactive.count' ),
          typeKey: 'APP.DASHBOARD.FLEET_OVERVIEW.INACTIVE',
          typeWithShipsKey: 'APP.DASHBOARD.FLEET_OVERVIEW.INACTIVE_SHIPS',
          className: 'lng-badge-inactivity'
        },
        {
          value: _.get( fleetOverview, 'unknown.count' ),
          typeKey: 'APP.DASHBOARD.FLEET_OVERVIEW.UNKNOWN',
          typeWithShipsKey: 'APP.DASHBOARD.FLEET_OVERVIEW.UNKNOWN_SHIPS',
          className: 'lng-badge-unknown'
        }
      ];

      vm.fleetOverviewReady = true;

      vm.gaugeConfig = {
        segments: segments,
        getSegmentText: $interpolate( '{{ value }}' ),
        getSegmentTooltipText: function( obj ) {
          return $interpolate( '{{ typeWithShipsKey | translate }}: {{::value }} / ' + vm.gaugeConfig.computedMaxValue )( obj );
        }
      };

    }

    function assignTenders( tenders ) {
      vm.requestTenders = true;
      vm.tenders = tenders;
    }

    function errorTenders() { vm.requestTenders = false; }
    function errorGaugeConfig() { vm.fleetOverviewReady = false; }
    function dashboardReady() { vm.dashboardReady = true; }
  }

} )();
