/**
 * SERVICE: DashboardService
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'DashboardService', DashboardService );

  function DashboardService( $q, Restangular ) {

    /**
     * Public methods exposed by this service
     * @type {{getFleetOverview: getFleetOverview, getLatestTenders: getLatestTenders, getInterestingMoves: getInterestingMoves}}
     */
    var factory = {
      getFleetOverview: getFleetOverview,
      getLatestTenders: getLatestTenders
    };

    return factory;

    ////////////

    function getFleetOverview() {
      var route = 'vessels/overview';
      return Restangular.one( route ).get();
    }

    function getLatestTenders() {

      var route = 'contracts/tenders';
      return Restangular.one( route )
          .getList()
          .then( extractPlain )
          .then( updateTenders );

      function updateTenders( data ) {
        if ( data ) {
          data.forEach( update );
        }

        return data;

        ////////////

        function update( tender ) {
          // convert string date to timestamps
          if ( tender.startDate ) {
            tender.$startDate = moment.utc( tender.startDate );
          }

          if ( tender.endDate ) {
            tender.$endDate = moment.utc( tender.endDate );
          }

          if ( tender.signedDate ) {
            tender.$signedDate = moment.utc( tender.signedDate );
          }
        }
      }
    }

    function extractPlain( data ) {
      return data.plain();
    }

  }

} )( this );
