/**
 * CONTROLLER: InstallationController
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .controller( 'InstallationController', InstallationController );

  function InstallationController(
    $ionicLoading,
    $ionicScrollDelegate,
    $ionicSlideBoxDelegate,
    $scope,
    $state,
    $stateParams,
    $timeout,
    InstallationService,
    MulticommoService
  ) {
    var vm = this,
      deregisterBeforeEnter;

    vm.slideHasChanged = slideHasChanged;
    vm.tabHasChanged = tabHasChanged;
    vm.isOil = MulticommoService.getCurrentCommodity().name === 'oil';

    enter();

    // Force back button for tab navigation
    deregisterBeforeEnter = $scope.$on( '$ionicView.beforeEnter', function( event, viewData ) {
      viewData.enableBack = true;
    } );

    $scope.$on( '$destroy', deregisterBeforeEnter );

    ////////////

    function enter() {

      vm.installationReady = false;
      $ionicLoading.show( {
        content: '',
        animation: 'fade-in'
      } );
      vm.installationPromise = InstallationService.getById( $stateParams.itemId )
        .then( assignInstallation )
        .then( updateIconClazz )
        .catch( errorRedirect )
        .finally( $ionicLoading.hide );

      ////////////

      function assignInstallation( installation ) {
        vm.installation = installation;
        return installation;
      }

      function updateIconClazz( installation ) {
        vm.iconClazz = global.clazzHelper.getItemClazz( global.installationHelper.getMainType( installation.type ) );
        vm.installationReady = true;
        return installation;
      }

      function errorRedirect() { $state.go( 'session.map' ); }
    }

    function slideHasChanged() {
      vm.slideIndex = $ionicSlideBoxDelegate.currentIndex();
      $ionicScrollDelegate.scrollTop();
      $timeout( $ionicScrollDelegate.resize, 50 );
    }

    function tabHasChanged( $index ) { $ionicSlideBoxDelegate.slide( $index ); }
  }
} )( this );
