/**
 * SERVICE: InstallationService
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */


( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'InstallationService', InstallationService );

  function InstallationService( $q, Restangular, ContractsService, MapCacheService, ActivityService ) {

    return {
      getById: getById
    };

    ////////////

    function getById( id, cancelPromise ) {

      var promiseBaseFromCache,
        promiseDetailsFromWeb,
        promiseContractsFromWeb,
        promisePC;

      promiseBaseFromCache = MapCacheService.getInstallationById( id );

      promiseDetailsFromWeb = Restangular.one( 'installations', id )
        .one( 'details' )
        .withHttpConfig( {
          timeout: cancelPromise
        } )
        .get()
        .then( extractPlain )
        .then( function( installation ) {
          return installation;
        } );

      promiseContractsFromWeb = ContractsService.getByPointer( Restangular.one( 'installations', id ) )
        .then( function( contracts ) {
          return {
            contracts: contracts
          };
        } );

      promisePC = ActivityService.load( id, 0, cancelPromise, 'installations' )
        .then( function( portCalls ) {
          return {
            portCalls: portCalls
          };
        } );

      return $q.all( [
        promiseDetailsFromWeb,
        promiseContractsFromWeb,
        promiseBaseFromCache,
        promisePC
      ] )
        .then( mergePromises )
        .then( getInstallationTypeText )
        .then( updateStatusText )
        .then( processData );


      ///////////////

      function extractPlain( data ) {
        return data.plain();
      }

      function mergePromises( promises ) {
        var destination,
          i,
          promise;

        if ( !promises || promises.length <= 0 ) {
          return;
        }

        destination = promises[ 0 ];

        for ( i = 1; i < promises.length; ++i ) {
          promise = promises[ i ];
          angular.extend( destination, promise );
        }
        return destination;
      }

      function getInstallationTypeText( installation ) {
        installation.$typeText = global.textTranslateHelper.translateType( installation.type );
        return installation;
      }

      function updateStatusText( installation ) {
        installation.$statusText = global.textTranslateHelper.translateStatus( installation.status );

        return installation;
      }

      function processData( data ) {
        data.statusClass = global.clazzHelper.getStatusClazz( data.status, data.resourceType );

        return data;
      }
    }

  }

} )( this );
