/**
 * CONTROLLER: PlayerController
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .controller( 'PlayerController', PlayerController );

  function PlayerController(
    $ionicLoading,
    $ionicSlideBoxDelegate,
    $ionicScrollDelegate,
    $scope,
    $timeout,
    $state,
    $stateParams,
    MulticommoService,
    PlayerService
  ) {
    var vm = this,
      deregisterBeforeEnter;

    vm.slideHasChanged = slideHasChanged;
    vm.tabHasChanged = tabHasChanged;
    vm.isOil = MulticommoService.getCurrentCommodity().name === 'oil';

    enter();

    // Force back button for tab navigation
    deregisterBeforeEnter = $scope.$on( '$ionicView.beforeEnter', function( event, viewData ) {
      viewData.enableBack = true;
    } );

    $scope.$on( 'destroy', deregisterBeforeEnter );

    ////////////

    function enter() {

      vm.playerReady = false;
      $ionicLoading.show( {
        content: '',
        animation: 'fade-in'
      } );
      vm.playerPromise = PlayerService.getById( $stateParams.itemId )
        .then( assignPlayer )
        .then( updateIconClazz )
        .catch( errorRedirect )
        .finally( $ionicLoading.hide );

      ////////////

      function assignPlayer( player ) {
        vm.player = player;
        return player;
      }

      function updateIconClazz( player ) {
        vm.iconClazz = global.clazzHelper.getItemClazz( player.type );
        vm.playerReady = true;
        return player;
      }

      function errorRedirect() {
        $state.go( 'session.map' );
      }
    }

    function slideHasChanged() {
      vm.slideIndex = $ionicSlideBoxDelegate.currentIndex();
      $ionicScrollDelegate.scrollTop();
      $timeout( $ionicScrollDelegate.resize, 50 );
    }

    function tabHasChanged( $index ) { $ionicSlideBoxDelegate.slide( $index ); }
  }
} )( this );
