/**
 * SERVICE: PlayerService
 */


( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'PlayerService', PlayerService );

  function PlayerService( $q, Restangular, ContractsService ) {
    var factory = {
      getById: getById
    };

    return factory;


    ////////////

    function getById( id, cancelPromise ) {

      var promiseDetailsFromWeb,
        promiseContractsFromWeb;

      promiseDetailsFromWeb = Restangular.one( 'players', id )
        .withHttpConfig( {
          timeout: cancelPromise
        } )
        .get()
        .then( extractPlain );

      promiseContractsFromWeb = ContractsService.getByPointer( Restangular.one( 'players', id ) )
        .then( function( contracts ) {
          return {
            contracts: contracts
          };
        } );

      return $q.all( [
        promiseDetailsFromWeb,
        promiseContractsFromWeb
      ] )
        .then( mergePromises )
        .then( getPlayerTypeText )
        .then( updateStatusText )
        .then( processData );


      ///////////////

      function extractPlain( data ) {
        return data.plain();
      }

      function mergePromises( promises ) {
        var destination,
          i,
          promise;

        if ( !promises || promises.length <= 0 ) {
          return;
        }

        destination = promises[ 0 ];

        for ( i = 1; i < promises.length; ++i ) {
          promise = promises[ i ];
          angular.extend( destination, promise );
        }

        return destination;
      }

      function getPlayerTypeText( player ) {
        player.$typeText = global.textTranslateHelper.translateType( player.type );
        return player;
      }

      function updateStatusText( player ) {
        player.$statusText = global.textTranslateHelper.translateStatus( player.status );

        return player;
      }

      function processData( data ) {
        data.statusClass = global.clazzHelper.getStatusClazz( data.status, data.resourceType );

        return data;
      }
    }
  }

} )( this );
