/**
 * CONTROLLER: SearchController
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .controller( 'SearchController', SearchController );

  function SearchController( $rootScope, $scope, $state, $stateParams, $ionicScrollDelegate, SearchService, TrackService ) {
    var vm = this,
      searchRequest,
      searchRequestListener = $rootScope.$on( 'search:change', function( event, request ) {
        if ( request ) {
          searchRequest = request;
          search( request );
        } else {
          assignHistory();
        }
      } );

    vm.results = {};
    vm.state = 'init';
    vm.select = select;
    vm.clearHistory = clearHistory;

    assignHistory();

    $scope.$on( '$destroy', function() {
      searchRequestListener();
    } );


    ////////////

    function assignHistory() {
      var i,
        historyLength;

      vm.history = SearchService.history.list();
      historyLength = ( vm.history ) ? vm.history.length : 0;

      if ( historyLength > 0 ) {
        vm.state = 'history';
        for ( i = 0; i < historyLength; i++ ) { displayItem( vm.history[ i ] ); }
      } else {
        vm.state = 'first-search';
      }

      ////////////

      function displayItem( item ) {
        item.$statusClazz = global.clazzHelper.getInactiveItemClazz( item.status );
        switch ( item.resourceType ) {
          case 'vessel':
            item.$iconClazz = 'fa-lng-vessel';
            break;
          case 'installation':
            item.$iconClazz = global.clazzHelper.getItemClazz( global.installationHelper.getMainType( item.type ) );
            break;
          default:
            break;
        }

        item.$name = ( item.fullName ? item.fullName : item.name );
      }
    }

    function clearHistory() {
      SearchService.history.clear();

      assignHistory();
    }

    function select( item ) {
      var type = item.resourceType;

      // Show history
      vm.history = SearchService.history.save( item );
      assignHistory();

      switch ( type ) {
        case 'vessel': {
          $state.go( 'session.map.vessel', {
            id: item.id,
            zoom: true
          }, {
            reload: true
          } );
          break;
        }
        default : {
          $state.go( 'session.' + item.resourceType, { itemId: item.id } );
          break;
        }
      }
    }

    function search( request ) {
      SearchService.search( request )
        .then( track )
        .then( searchSuccess )
        .catch( searchError );


      ////////////

      function track( data ) {
        var title = 'Search "' + searchRequest + '".',
          tracking = {
            route: 'search',
            request: searchRequest,
            title: title
          };

        angular.forEach( data, function( value, key ) {
          tracking.title = tracking.title + ' ' + value.length + ' ' + key + '(s).';
          tracking[ key + 's_count' ] = value.length;
        } );

        TrackService.track( tracking );

        return data;
      }

      function searchSuccess( data ) {

        $ionicScrollDelegate.scrollTop();

        vm.results = data;
        vm.types = [ 'vessel', 'installation', 'zone', 'player' ];

        if ( hasResultsProcess( data ) ) {

          angular.forEach( vm.results, function( entity, type ) {

            angular.forEach( entity, function( item ) {
              item.$iconClazz = type === 'installation' ? global.clazzHelper.getItemClazz( global.installationHelper.getMainType( item.type ) ) : 'fa-lng-' + type;
              item.$name = ( item.fullName ) ? item.fullName : item.name;
              item.$statusClazz = global.clazzHelper.getInactiveItemClazz( item.status );
            } );
          } );

          vm.state = 'results';

        } else {
          vm.state = 'empty';
        }

        return data;


        ////////////

        function hasResultsProcess( results ) {
          if ( !results ) { return false; }
          if ( results.zone && results.zone.length > 0 ) { return true; }
          if ( results.vessel && results.vessel.length > 0 ) { return true; }
          if ( results.installation && results.installation.length > 0 ) { return true; }
          if ( results.player && results.player.length > 0 ) { return true; }
          return false;
        }
      }

      function searchError() { vm.state = 'empty'; }
    }

  }

} )( this );
