/**
 * SERVICE: SearchService
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'SearchService', SearchService );

  function SearchService( $q, Restangular, SettingsService ) {

    // 8 : max number of visible items for iPhone 4
    var HISTORY_MAX_SIZE = 8,

      factory = {
        search: search,
        searchVC: searchVC,
        history: {
          list: historyList,
          save: historySave,
          clear: historyClear
        }
      };

    return factory;


    ////////////

    // For search.html
    function search( request ) {
      var searchRequest = {
        q: request
      };
      return Restangular.one( 'search' ).get( searchRequest )
        .then( extractResults );
    }

    // Only for voyage calculator
    function searchVC( request, viapoints, types ) {
      var searchRequest = {
        q: request,
        viapoints: viapoints
      };

      return Restangular.one( 'search' ).get( searchRequest )
        .then( extractResults )
        .then( function( xs ) { return filterValidResults( xs, types ); } );


      ////////////

      function filterValidResults( results, types ) {
        return _( results )
          .pick( types )
          .values()
          .value();
      }
    }

    function extractResults( data ) {
      if ( data.results ) {
        return data.results;
      } else {
        return $q.reject();
      }
    }

    function historyList() {
      var rawHistory = localStorage.getItem( 'searchHistory' );
      if ( rawHistory ) { return JSON.parse( rawHistory ).filter( function( x ) { return x.context === SettingsService.getSettings().COMMODITY; } ); }
    }

    function historySave( item ) {

      var rawHistory = localStorage.getItem( 'searchHistory' ),
        history,
        overflow;

      item.context = SettingsService.getSettings().COMMODITY;

      if ( rawHistory ) {
        history = JSON.parse( rawHistory );

        // remove old entry
        _.remove( history, function( n ) {
          return n.id === item.id && n.resourceType === item.resourceType;
        } );

        // slice overflow
        overflow = history.length - HISTORY_MAX_SIZE + 1;
        if ( overflow > 0 ) {
          history = history.slice( overflow );
        }

      } else {
        history = [];
      }

      history.push( item );

      localStorage.setItem( 'searchHistory', JSON.stringify( history ) );

      return history;
    }

    function historyClear() {
      localStorage.removeItem( 'searchHistory' );
    }
  }

} )();
