/**
 * CONTROLLER: TeamboardController
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .controller( 'TeamboardController', TeamboardController );

  function TeamboardController( $ionicLoading, $scope, $rootScope, $filter, $timeout, $ionicScrollDelegate, LoginService, TeamboardService ) {

    var vm = this,
      timer,
      auth = LoginService.getAuth(),

      // Scroll bottom on new message
      destroyTBmsg = $rootScope.$on( 'teamboard:newmessage', scrollBottom );

    vm.newmessage = '';
    vm.messages = [];
    vm.teamboardReady = false;

    // Organisation name
    vm.organisationName = LoginService.getAuth().user.organisation.name;
    vm.sendMessage = sendMessage;
    vm.isNewMessageValid = isNewMessageValid;
    vm.scrollBottom = scrollBottom;

    init();

    $scope.$on( '$destroy', function() {
      // Remove listeners
      destroyTBmsg();
      $timeout.cancel( timer );

      // Close SSE session
      TeamboardService.disconnect();

    } );

    ////////////

    function init() {

      $ionicLoading.show( {
        content: '',
        animation: 'fade-in'
      } );

      // Get fresh messages
      TeamboardService.getMessages()
        .then( function( data ) {
          vm.teamboardReady = true;
          vm.messages = data;
          scrollBottom();
        } )
        .finally( $ionicLoading.hide );

      // Start SSE session
      TeamboardService.connect( auth );

    }

    function sendMessage() {
      var filterTranslate = $filter( 'translate' ),
        text;

      if ( !vm.newmessage ) {
        return;
      }

      vm.newmessage = vm.newmessage.trim();

      if ( vm.newmessage.length <= 0 ) {
        return;
      }

      // Test Condition to improve
      if ( vm.newmessage.indexOf( '</a>' ) > -1 ) {
        text = filterTranslate( 'APP.TEAMBOARD.POST_ERROR_HYPERLINK' );
        $scope.$emit( 'toast:error', text );
      } else {
        TeamboardService.postMessage( vm.newmessage )
          .then( cleanNewMessage )
          .catch( errorPostMessage );
      }

      ////////////

      function cleanNewMessage() {
        vm.newmessage = '';
      }

      function errorPostMessage() {
        $scope.$emit( 'toast:error', 'Cannot post message' );
      }
    }

    function scrollBottom() {
      timer = $timeout( function() {
        $ionicScrollDelegate.scrollBottom( false );
      }, 200 );
    }


    function isNewMessageValid() {
      return vm.newmessage && vm.newmessage.trim().length > 0;
    }

  }

} )();
