/**
 * CONTROLLER: VesselController
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .controller( 'VesselController', VesselController );

  function VesselController(
    $ionicLoading,
    $ionicSlideBoxDelegate,
    $ionicScrollDelegate,
    $scope,
    $timeout,
    $state,
    $stateParams,
    VesselService
  ) {
    var vm = this,
      deregisterBeforeEnter;

    vm.slideHasChanged = slideHasChanged;
    vm.tabHasChanged = tabHasChanged;
    vm.vesselReady = false;

    enter();

    // Force back button for tab navigation
    deregisterBeforeEnter = $scope.$on( '$ionicView.beforeEnter', function( event, viewData ) {
      viewData.enableBack = true;
    } );

    $scope.$on( 'destroy', deregisterBeforeEnter );


    ////////////

    function enter() {

      $ionicLoading.show( {
        content: '',
        animation: 'fade-in'
      } );
      vm.vesselPromise = VesselService.getById( $stateParams.itemId )
        .then( assignVessel )
        .catch( errorRedirect )
        .finally( $ionicLoading.hide );

      ////////////

      function assignVessel( vessel ) {
        vm.vessel = vessel;
        vm.vesselReady = true;
        return vessel;
      }

      function errorRedirect() {
        $state.go( 'session.map' );
      }
    }

    function slideHasChanged() {
      vm.slideIndex = $ionicSlideBoxDelegate.currentIndex();
      $ionicScrollDelegate.scrollTop();
      $timeout( $ionicScrollDelegate.resize, 50 );
    }

    function tabHasChanged( $index ) { $ionicSlideBoxDelegate.slide( $index ); }
  }
} )();
