/**
 * CONTROLLER: VoyageCalculatorController
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .controller( 'VoyageCalculatorController', VoyageCalculatorController );

  function VoyageCalculatorController( $scope, $state, LoginService, VoyageCalculatorService, TrackService ) {
    var vm = this;

    vm.canSeeVC = LoginService.canSeeVoyageCalculator();
    vm.calculateVoyage = calculateVoyage;
    vm.findOrigin = VoyageCalculatorService.getOriginSearchResults;
    vm.findViaPoints = VoyageCalculatorService.getViaPoints;
    vm.findDestinations = VoyageCalculatorService.getDestinationSearchResults;
    vm.addViaPoint = addViaPoint;
    vm.deleteViaPoint = deleteViaPoint;
    vm.optionValue = optionValue;
    vm.data = VoyageCalculatorService.backup;
    vm.data.speed = vm.data.speed || 17;
    vm.clear = clear;

    ////////////

    function calculateVoyage() {
      if ( vm.isLoading ) { return; }

      vm.isLoading = true;

      VoyageCalculatorService.calculate( vm.data.origin, vm.data.destination, vm.data.speed, vm.data.viaPoints )
        .then( track )
        .then( assignVoyageCalculator )
        .then( showResultInfo )
        .catch( showError )
        .finally( function() { vm.isLoading = false; } );

      ////////////

      function track( result ) {
        var data = {
          route: 'voyage_calculator',
          origin_type: vm.data.origin.resourceType,
          origin_id: vm.data.origin.id,
          origin_name: vm.data.origin.name,
          destination_type: vm.data.destination.resourceType,
          destination_id: vm.data.destination.id,
          destination_name: vm.data.destination.name,
          distance: result.distance,
          device: 'mobile'
        };

        if ( vm.data.speed ) { data.speed = vm.data.speed; }
        TrackService.track( data );
        return result;
      }

      function assignVoyageCalculator( result ) {
        vm.result = result;
        return result;
      }

      function showResultInfo( result ) {
        $state.go( 'session.map' );
        return result;
      }


      function showError( e ) {
        $scope.$emit( 'toast:error', 'Voyage estimation failed. ' + e.statusText + ' (' + e.status + ')' );
      }
    }

    function addViaPoint() { vm.data.viaPoints.push( { res: null } ); }

    function deleteViaPoint( vp ) {
      var i = vm.data.viaPoints.indexOf( vp );
      if ( i >= 0 ) { vm.data.viaPoints.splice( i, 1 ); }
    }

    function optionValue( item ) {
      return '<i class="icon fa ' + global.itemIconHelper.get( item ) + '"></i> ' + item.name;
    }

    function clear( action /* :string */, index /* :?number */ ) /* :void */ {
      switch ( action ) {
        case 'origin':
        case 'destination':
        case 'speed':
          vm.data[ action ] = null;
          break;
        case 'viaPoint':
          if ( _.get( vm, 'data.viaPoints[' + index + ']' ) ) {
            vm.data.viaPoints[ index ].res = null;
          }
          break;
        default:
          console.info( 'No property ' + action + ' to be cleared.' );
      }
    }

  }
} )( this );
