/**
 * CONTROLLER: ZoneController
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .controller( 'ZoneController', ZoneController );

  function ZoneController(
    $ionicLoading,
    $ionicSlideBoxDelegate,
    $ionicScrollDelegate,
    $scope,
    $timeout,
    $state,
    $stateParams,
    MulticommoService,
    ZoneService
  ) {
    var vm = this,
      deregisterBeforeEnter;

    vm.slideHasChanged = slideHasChanged;
    vm.tabHasChanged = tabHasChanged;
    vm.isOil = MulticommoService.getCurrentCommodity().name === 'oil';

    enter();

    // Force back button for tab navigation
    deregisterBeforeEnter = $scope.$on( '$ionicView.beforeEnter', function( event, viewData ) {
      viewData.enableBack = true;
    } );

    $scope.$on( 'destroy', deregisterBeforeEnter );

    ////////////

    function enter() {

      vm.zoneReady = false;
      $ionicLoading.show( {
        content: '',
        animation: 'fade-in'
      } );
      vm.zonePromise = ZoneService.getById( $stateParams.itemId )
        .then( assignZone )
        .then( updateIconClazz )
        .catch( errorRedirect )
        .finally( $ionicLoading.hide );

      ////////////

      function assignZone( zone ) {
        vm.zone = zone;
        vm.linkedInstallations = zone.import;
        return zone;
      }

      function updateIconClazz( zone ) {
        vm.iconClazz = global.clazzHelper.getItemClazz( zone.type );
        vm.zoneReady = true;
        return zone;
      }

      function errorRedirect() {
        $state.go( 'session.map' );
      }
    }

    function slideHasChanged() {
      vm.slideIndex = $ionicSlideBoxDelegate.currentIndex();
      $ionicScrollDelegate.scrollTop();
      $timeout( $ionicScrollDelegate.resize, 50 );
    }

    function tabHasChanged( $index ) { $ionicSlideBoxDelegate.slide( $index ); }
  }
} )( this );
