/**
 * CONTROLLER: StartController
 * Entry point of the application
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .controller( 'StartController', StartController );

  function StartController( $q, $location, $state, $ionicHistory, LoginService, NetworkInformationService ) {

    start();

    ////////////

    function start() {

      var credentials = LoginService.getOrCreateClearCredentials(),
        urlFromScheme;

      $ionicHistory.nextViewOptions( {
        disableAnimate: true,
        disableBack: true
      } );

      // On Android,
      // the Connection object provided by cordova-plugin-network-information
      // and used in the isOnline method may not be available when isOnline method is called.
      // Ugly but adding a small timeout fixes this.
      // @see: http://bit.ly/1Ujd5D3
      setTimeout( function() {
        var waitPromise;

        if ( NetworkInformationService.isOnline() ) {
          if ( !credentials.email ) {
            // Add some delay to avoid jumping too fast to the login page (UX purpose)
            waitPromise = waitFor( 200 );
            waitPromise.then( function() { $state.go( 'login' ); } );
          } else {
            // Add some minimal delay to avoid jumping too fast to the new state (UX purpose)
            // @todo: Add promises here to load google map script and mapItems data
            // @todo: Update the message displayed to warn the user if something went wrong
            waitPromise = waitFor( 1500 );

            //App can be started safely from here
            // - Start the app on map state or to a state defined by custom_scheme
            $q.all( [ waitPromise ] )
              .then( function() {
              // If opening the app from a notification and that an url is specified, redirect to this url
              // Otherwise, show the map
                urlFromScheme = localStorage.getItem( 'urlFromScheme' );
                if ( urlFromScheme ) {
                  localStorage.removeItem( 'urlFromScheme' );
                  $location.url( urlFromScheme );
                } else {
                  $state.go( 'session.map' );
                }
              } );
          }
        } else {
          $state.go( 'offline' );
        }
      }, 600 );

      ////////////

      function waitFor( duration ) {
        var defer = $q.defer();
        setTimeout( function() { defer.resolve(); }, duration );
        return defer.promise;
      }

    }
  }

} )();
