/**
 * HELPER clazz
 * @author Jin
 */

( function( global ) {
  'use strict';

  global.clazzHelper = ( global.module || {} ).exports = {
    getStatusClazz: getStatusClazz,
    getItemClazz: getItemClazz,
    getInactiveItemClazz: getInactiveItemClazz,
    getTimeInfoClazz: getTimeInfoClazz,
    getTypeClazz: getTypeClazz
  };


  ////////////

  // lng badge clazz for item status
  function getStatusClazz( status, type ) {
    if ( !status ) {
      return;
    }

    switch ( status ) {
      case 'Active': {
        return 'lng-badge-' + type + '-inactivity';
      }

      case 'Speculative':
      case 'Approved':
      case 'Under Construction':
      case 'Planned and Proposed': {
        return 'lng-badge-' + type + '-underconstruction';
      }

      case 'Suspended':
      case 'Decommissioned':
      case 'Inactive': {
        return 'lng-badge-' + type + '-inactive';
      }
    }

  }

  function getInactiveItemClazz( status ) {
    switch ( status ) {
      case 'Active':
      case '': {
        return;
      }
      case 'Speculative':
      case 'Approved':
      case 'Under Construction':
      case 'Planned and Proposed': {
        return 'lng-badge-underconstruction';
      }

      case 'Suspended':
      case 'Decommissioned':
      case 'Inactive': {
        return 'lng-badge-inactive';
      }
    }
  }

  function getItemClazz( type ) {
    switch ( type ) {
      case 'Export': {
        return 'fa-lng-export';
      }
      case 'Import': {
        return 'fa-lng-import';
      }

      case 'Shipyard': {
        return 'fa-lng-shipyard';
      }

      case 'Anchorage': {
        return 'fa-lng-anchorage';
      }

      case 'Player': {
        return 'fa-lng-player';
      }

      case 'Vessel': {
        return 'fa-lng-vessel';
      }

      default: {
        return 'fa-globe';
      }
    }
  }

  /**
   * Return a class name that will be used in a template
   * @param {string} (format: 2014-01-01T00:00:00.000+0000) open --> start date
   * @param {string} (format: 2014-01-01T00:00:00.000+0000) until --> end date
   * @returns {string} a string value among values ['past','current','future']
   */
  function getTimeInfoClazz( open, until ) {
    var now = moment.utc(),
      start = open ? moment.utc( open ) : null,
      end = until ? moment.utc( until ) : null;

    if ( start && start < now && ( !end || now < end ) ) {
      return 'current';
    }

    if ( end && end < now ) {
      // past : end < now
      return 'past';
    }

    if ( ( start && now < start ) || ( !start && !end ) ) {
      // future : now < start or both start and end are unknown
      return 'future';
    }

    return '';
  }

  /**
   * Return a class name to according to the type of contract
   * @param {string} type, type of contract (tender, spot ...)
   * @returns {string} - A class name
   */
  function getTypeClazz( type ) {
    switch ( type ) {
      case 'Spot':
      case 'Stripe':
      case 'Tender': {
        return 'lng-badge-trade-shortterm';
      }
      default: {
        return 'lng-badge-trade-longterm';
      }
    }
  }

} )( this );
