/**
 * SERVICE: NetworkInformationService
 * A service allowing to use cordova-plugin-network-information along the application
 *
 * @see: https://github.com/apache/cordova-plugin-network-information
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'NetworkInformationService', NetworkInformationService );

  function NetworkInformationService( $rootScope, $state, $window ) {

    var stateBeforeDisconnect = 'start',

      factory = {
        isOnline: isOnline,
        hasBeenReconnected: hasBeenReconnected,
        hasBeenDisconnected: hasBeenDisconnected
      };

    return factory;

    ////////////

    /**
     * Check current connection status
     * Useful when starting the app to know if it is worth the value to start requests
     * This requires cordova-plugin-network-information
     * If not available (such as in desktop browser during development), return true
     */
    function isOnline() {
      console.info( 'Checking user connection status.' );
      if ( !navigator.connection ) {
        return true;
      } else if ( window.Connection && ( navigator.connection.type === window.Connection.NONE || navigator.connection.type === window.Connection.UNKNOWN ) ) {
        hasBeenDisconnected();
      } else {
        return true;
      }
    }

    /**
     * This is the callback function used when the 'online' event is fired
     * Now that the network is back, we load the latest known state before disconnection
     */
    function hasBeenReconnected() {
      console.info( 'Network ON. User has been reconnected.' );
      if ( !stateBeforeDisconnect || stateBeforeDisconnect === 'start' ) {
        $rootScope.$apply( function() {
          $state.go( 'start' );
          $window.location.reload();
        } );
      } else {
        $state.go( stateBeforeDisconnect );

        // For teamboard, reload the state, to restart SSE
        if ( stateBeforeDisconnect === 'session.teamboard' ) {
          $window.location.reload();
        }
      }
    }

    /**
     * This is the callback function used when the 'offline' event is fired
     * We save a reference to the latest known state before the disconnection
     */
    function hasBeenDisconnected() {
      console.info( 'Network OFF. User has been disconnected.' );
      /**
       * Sometimes, 'offline' event fire twice
       * When it is the case last state known saved becomes 'offline'
       * so that user is blocked to 'offline' state when 'online' event is fired
       * As a consequence, last state must be affected to stateBeforeDisconnect unless it matches 'offline'
       */
      if ( $state.current.name !== 'offline' ) {
        stateBeforeDisconnect = $state.current.name;
        $state.go( 'offline' );
      }

    }
  }

} )();
