/**
 * INTERCEPTOR: Token
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .config( TokenInterceptor );

  function TokenInterceptor( $httpProvider ) {
    var replays = [],
      loginPromise;

    $httpProvider.interceptors.push( interceptorFunc );


    ////////////

    function interceptorFunc( $injector, $q ) {
      var interceptor = { responseError: responseError };
      return interceptor;

      ////////////

      function responseError( rejection ) {
        var status = rejection.status,
          deferred = $q.defer(),
          replay;

        if ( status === 0 || status === 401 ) {

          // Close SSE feed opened before receiving the 401 error
          // to avoid spamming 401 errors...
          $injector.get( 'TeamboardService' ).disconnect();

          replay = {
            success: function() {
              $injector.get( '$http' )( rejection.config )
                .then( function( response ) {
                  deferred.resolve( response );
                }, function( error ) {
                  deferred.reject( error );
                } );
            },
            cancel: function() { deferred.reject( rejection ); }
          };

          replays.push( replay );

          if ( !loginPromise ) {
            loginPromise = $injector.get( 'LoginService' ).loginWithCredentials()
              .then( clearLoginPromise )
              .then( replayRequests )
              .catch( cancelRequestsAndRedirect );
          }
        } else {
          deferred.reject( rejection );
        }

        return deferred.promise;


        ////////////

        function clearLoginPromise( auth ) {
          loginPromise = undefined;
          return auth;
        }

        function replayRequests( auth ) {
          var i = 0,
            len = replays.length;

          for ( i; i < len; i++ ) { replays[ i ].success(); }
          replays.length = 0;
          return auth;
        }

        function cancelRequestsAndRedirect() {
          var i = 0,
            len = replays.length;

          for ( i; i < len; i++ ) { replays[ i ].cancel(); }
          replays.length = 0;
          $injector.get( '$state' ).go( 'login' );
          return $q.resolve();
        }
      }
    }
  }

} )();
