/**
 * ROUTE: Redirect on secure route
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .run( TokenRouting );

  function TokenRouting( $location, $rootScope, $urlRouter, LoginService ) {
    // Every time a state change occurs somewhere in the application, check that the user is logged in
    // If he isn't, prevent the current state change from happening and redirect to state 'login'
    var destroyStateChange = $rootScope.$on( '$stateChangeStart', function( event, toState ) {
      var requireAuth,
        auth;

      if ( toState.data ) { requireAuth = toState.data.requireAuth; }

      if ( requireAuth ) {
        auth = LoginService.getAuth();

        if ( !auth ) {
          // Prevent current state change from happening
          event.preventDefault();

          // resume current state change
          /*
           * @note using $state#go here is buggy and causes an angular $digest loop, even though
           * we used event.preventDefault. Apparently, this is due to poor interaction between
           * $urlRouterProvider#otherwise and ui-router:
           * "Bumped into the same bug yesterday. If you get into the $startStateChange handler from $urlRouterProvider.otherwise URL and call preventDefault(), you end up in an infinite loop"
           * https://github.com/angular-ui/ui-router/issues/600
           */
          LoginService.loginWithCredentials()
            .then( function() { $urlRouter.sync(); } )
            .catch( function() { $location.path( '/login' ); } );
        }
      }
    } );

    $rootScope.$on( '$destroy', destroyStateChange );
  }
} )();
