/**
 * DIRECTIVE: activity
 * MB: We should rewrite this to avoid using self = this along the component.
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .directive( 'activity', activity );

  function activity( $stateParams, TrackService, ActivityService ) {

    var page,
      self,
      directive;

    directive = {
      restrict: 'E',
      scope: {},
      bindToController: {
        promise: '=',
        type: '@'
      },
      templateUrl: 'components/activity/activity.html',
      controller: ActivityController,
      controllerAs: 'vm'
    };

    ////////////

    /* @ngInject */
    function ActivityController( $scope ) {

      self = this;
      page = 0;

      // Reference to $scope to be able to broadcast infinite-scroll event
      this.scope = $scope;
      this.entity = null;
      this.portCallsByYear = [];
      this.loadMoreVisible = true;
      this.isReady = false;
      this.promise
        .then( track )
        .then( this.loadMore );

      ////////////

      function track( entity ) {
        var title = self.type + ' ' + entity.name + ' (' + entity.id + ') - Activity',
          trackObj = {
            route: self.type + '/activity',
            title: title
          };
        trackObj[ self.type + '_id' ] = entity.id;
        trackObj[ self.type + '_name' ] = entity.name;

        TrackService.track( trackObj );

        return entity;
      }
    }

    /**
     *
     * @returns {number}
     */
    ActivityController.prototype.countPortCalls = function countPortCalls() {
      var count = 0,
        portCallsByYear = self.portCallsByYear,
        i = 0,
        len = self.portCallsByYear.length;

      for ( i; i < len; i++ ) {
        count += portCallsByYear[ i ].length;
      }

      return count;
    };

    /**
     * loadMore
     */
    ActivityController.prototype.loadMore = function loadMore() {

      var promise,
        type = self.type + 's';

      if ( page === 0 && type !== 'zones' ) {
        promise = self.promise.then( function( data ) {
          self.entity = data;
          return data.portCalls;
        } );

      } else {
        promise = ActivityService.load(
          $stateParams.itemId,
          page,
          undefined,
          type
        );

      }

      page++;
      promise
        .then( addPortCalls )
        .then( notifyInfiniteScrollComplete );

      ////////////

      function addPortCalls( portCalls ) {
        var i = 0,
          len = portCalls.length;

        for ( i; i < len; i++ ) {
          addPortCall( portCalls[ i ] );
        }

        self.loadMoreVisible = portCalls.length > 0;

        return portCalls;


        ////////////

        function addPortCall( portCall ) {

          var yearPortCalls;

          if ( !portCall.year ) {
            return;
          }

          yearPortCalls = _.find( self.portCallsByYear, function( pc ) {
            return pc.year === portCall.year;
          } );

          if ( !yearPortCalls ) {
            yearPortCalls = {
              year: portCall.year,
              portCalls: []
            };
            self.portCallsByYear.push( yearPortCalls );
          }

          yearPortCalls.portCalls.push( portCall );
        }
      }

      function notifyInfiniteScrollComplete() {
        self.scope.$broadcast( 'scroll.infiniteScrollComplete' );
        self.isReady = true;
      }
    };

    return directive;

  }

} )();
