/**
 * SERVICE: ActivityService
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'ActivityService', ActivityService );

  function ActivityService( $window, Restangular ) {

    var factory = {
      load: load
    };

    return factory;


    ////////////

    function load( id, page, cancelPromise, type ) {
      var items,
        pageSize,
        promise;

      items = {
        minimumRequested: 25,
        // A row height
        pixelHeight: 25
      };

      pageSize = Math.max(
        items.minimumRequested,
        Math.round( $window.innerHeight / items.pixelHeight )
      );

      promise = Restangular.one( type, id );

      if ( cancelPromise ) {
        promise = promise.withHttpConfig( {
          timeout: cancelPromise
        } );
      }

      return promise.getList( 'portcalls', {
        from: page * pageSize,
        size: pageSize,
        withForecasted: false
      } )
        .then( extractPlain )
        .then( updatePortCalls );


      ////////////

      function extractPlain( data ) {
        return data.plain();
      }

      function updatePortCalls( portCalls ) {
        var i,
          len = portCalls.length;

        for ( i = 0; i < len; i++ ) {
          update( portCalls[ i ] );
        }

        return portCalls;


        ////////////

        /**
         * Enrich PC data
         * @param portCall
         * @todo: Simplify this method --> eslint complexity warning
         */
        function update( portCall ) {

          var j,
            pcLength;

          portCall.arrival = portCall.start;
          portCall.departure = portCall.end;

          if ( portCall.future === undefined ) {
            portCall.future = ( portCall.eta && !portCall.arrival );
          }
          if ( portCall.eta ) {
            portCall.eta = moment.utc( portCall.eta );
            portCall.year = portCall.eta.year();
          }
          if ( portCall.rawEta ) {
            portCall.rawEta = moment.utc( portCall.rawEta );
            portCall.isETAOverrided = !portCall.rawEta.isSame( portCall.eta );

          }
          if ( portCall.calculatedEta ) {
            portCall.calculatedEta = moment.utc( portCall.calculatedEta );
            portCall.diffETA = portCall.calculatedEta.diff( portCall.eta, 'days' );
          }

          if ( portCall.arrival ) {
            portCall.arrival = moment.utc( portCall.arrival );
            if ( !portCall.year ) {
              portCall.year = portCall.arrival.year();
            }
          }
          if ( portCall.departure ) {
            portCall.departure = moment.utc( portCall.departure );
            if ( !portCall.year ) {
              portCall.year = portCall.departure.year();
            }
          }

          portCall.dateHuman = global.datesUtils.momentShortHumanDate( portCall.arrival ? portCall.arrival : portCall.eta );

          if ( portCall.installation && portCall.installation.type ) {
            portCall.installationTypeIcon = global.itemIconHelper.get( portCall.installation );
          } else {
            portCall.installationTypeIcon = 'fa-lng-defaulticon';
          }

          if ( portCall.confidence ) {
            portCall.confidenceHuman = Math.round( portCall.confidence * 100 ) + '%';
          }

          if ( portCall.forecastedTree ) {
            pcLength = portCall.forecastedTree;
            for ( j = 0; j < pcLength; j++ ) {
              update( portCall.forecastedTree[ j ] );
            }
          }

          if ( portCall.children ) {
            pcLength = portCall.children;
            for ( j = 0; j < pcLength; j++ ) {
              update( portCall.children[ j ] );
            }
          }
        }
      }
    }

  }

} )( this );
