/**
 * SERVICE: ContractsService
 */

( function() {
  'use strict';

  angular
    .module( 'lng' )
    .factory( 'ContractsService', ContractsService );

  function ContractsService() {

    return {
      getByPointer: getByPointer,
      formatContracts: formatContracts,
      formatContract: formatContract
    };

    ////////////

    /**
     * Returns contracts sorted and grouped by type
     * @param pointer
     * @returns {*}
     */
    function getByPointer( pointer ) {
      return pointer
        .all( 'contracts' )
        .getList()
        .then( function( contracts ) {
          return contracts.plain();
        } )
        .then( formatContracts );
    }

    /**
     * Process API response to format contract structure to Frontend needs
     * @param contracts
     * @returns {*}
     */
    function formatContracts( contracts ) {

      return _.chain( contracts )
        .forEach( formatContract )
        .orderBy( [ '$isOver', '$noEndDate', '$noStartDate', '$endDate', '$startDate' ], [ 'asc', 'asc', 'asc', 'desc', 'desc' ] )
        .groupBy( 'type' )
        .value();
    }

    /**
     * Add some keys that will be used by the controller and the view
     * @param contract
     */
    function formatContract( contract ) {
      var now = new Date();

      // Following keys are used to order contracts
      contract.$isOver = false;
      contract.$noStartDate = false;
      contract.$noEndDate = false;

      // convert string date to timestamps
      if ( contract.startDate ) {
        contract.$startDate = moment.utc( contract.startDate );
      } else {
        contract.$noStartDate = true;
      }

      if ( contract.endDate ) {
        contract.$endDate = moment.utc( contract.endDate );

        if ( contract.$endDate.isBefore( now ) ) {
          contract.$isOver = true;
        }

      } else {
        contract.$noEndDate = true;
      }

      if ( contract.signedDate ) {
        contract.$signedDate = moment.utc( contract.signedDate );
      }

      return contract;
    }
  }

} )();
