/**
 * DIRECTIVE: info
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .directive( 'info', info );

  function info() {
    var directive = {
      restrict: 'E',
      scope: {},
      bindToController: {
        promise: '=',
        type: '@'
      },
      template: '<ng-include src="vm.getTemplateUrl()"/>',
      controller: InfoController,
      controllerAs: 'vm'
    };

    return directive;

    ////////////

    /* @ngInject */
    function InfoController( TrackService ) {

      var self = this;

      this.getTemplateUrl = getTemplateUrl;

      this.promise
        .then( track )
        .then( clearPosition )
        .then( assignEntity );

      ////////////

      function getTemplateUrl() {
        if ( this.type === 'zone.import' || this.type === 'zone.export' ) {
          return 'components/info/zone-info.html';
        }
        return 'components/info/' + this.type + '-info.html';
      }

      function track( entity ) {
        var title = self.type + ' ' + entity.name + ' (' + entity.id + ') - Info',
          trackObj = {
            route: self.type + '/info',
            title: title
          };
        trackObj[ self.type + '_id' ] = entity.id;
        trackObj[ self.type + '_name' ] = entity.name;

        TrackService.track( trackObj );

        return entity;
      }

      function clearPosition( entity ) {
        if ( self.type === 'installation' && !entity.position ) {
          entity.position = {
            shorePosition: '-',
            geoZone: '-',
            address: '-'
          };
        }

        return entity;
      }

      function assignEntity( entity ) {
        self.entity = entity;
        if ( self.type === 'zone.import' ) {
          self.installationType = 'import';
          self.linkedInstallations = entity.import;
        }

        if ( self.type === 'zone.export' ) {
          self.installationType = 'export';
          self.linkedInstallations = entity.export;
        }

        return entity;
      }

    }
  }

} )();
