/**
 * A Service / Module
 * to transform data from the API related to the map in a format that suits Frontend needs
 */

( function() {
  'use strict';

  angular
    .module( 'lng' )
    .factory( 'MapApiTransformsService', MapApiTransformsService );

  function MapApiTransformsService() {

    var factory;

    factory = {
      formatPosition: formatPositionFromAPI,
      formatInstallationType: formatInstallationType
    };

    return factory;

    ////////////

    /**
     * For positions / coordinates, Google Maps format should be the only one used by Angular code
     * API send right now heterogeneous format.
     * This method must be used to clean position format
     * @param {Object} position object from API
     * @returns { lat: <Number>, lng: <Number> } formattedPosition compliant with Google Maps format
     */
    function formatPositionFromAPI( position ) {

      var formattedPosition = {};

      if ( !position ) {
        return null;
      }

      // If this method is called on an object already formatted,
      // returns the already formatted data --> identity function.
      if ( position.$geo ) {
        return position.$geo;
      }

      // Otherwise,
      // check the different known formats from API
      // and returns the expected one {lat: <number>, lng: <number>}
      formattedPosition.lat = parseFloat( position.lat
        || position.latitude
        || ( position.geo && position.geo.lat )
        || ( position.geo && position.geo.latitude ), 10 );

      formattedPosition.lng = parseFloat( position.lon
        || position.lng
        || position.longitude
        || ( position.geo && position.geo.lon )
        || ( position.geo && position.geo.lng )
        || ( position.geo && position.geo.longitude ), 10 );

      return ( formattedPosition.lat && formattedPosition.lng ) ? formattedPosition : null;

    }

    /**
     * Method to ensure that installation type use Import / Export terminology along the web app rather than Regasification / Liquefaction one.
     * Final term displayed on the screen depends on commodity and is handled by MulticommoService.
     * @param term
     * @returns {*}
     */
    function formatInstallationType( term ) {
      return ( term && term.replace( 'Regasification', 'Import' ).replace( 'Liquefaction', 'Export' ) );
    }

  }

} )();
