( function( global ) {

  'use strict';

  var Tooltip;

  global.newMapTooltip = newMapTooltip;

  ////////////

  function newMapTooltip( gm, opts ) {
    var TooltipClass = getTooltipClass( gm );
    return new TooltipClass( opts );
  }

  function getTooltipClass( gm ) {
    if ( !Tooltip ) {

      Tooltip = function tooltip( opts ) {
        this.marker_ = opts.marker;
        this.content = opts.content;
        this.cssClass_ = 'map-tooltip ';
        this.clickListeners = [];
        this.div_ = null;
        if ( opts.className ) {
          this.cssClass_ += opts.className;
        }
      };

      Tooltip.prototype = new gm.OverlayView();

      Tooltip.prototype.onAdd = function() {

        var i = 0,
          len = this.clickListeners.length,
          div = document.createElement( 'DIV' );

        div.style.position = 'absolute';

        if ( this.cssClass_ ) {
          div.className += ' ' + this.cssClass_;
        }

        div.innerHTML = this.content;

        this.div_ = div;

        for ( i; i < len; i++ ) {
          gm.event.addDomListener( div, 'click', this.clickListeners[ i ] );
        }

      };

      Tooltip.prototype.draw = function() {
        this.show();
      };

      Tooltip.prototype.setMap = function( map ) {
        gm.OverlayView.prototype.setMap.call( this, map );

        this._map = map;
      };

      Tooltip.prototype.updateDraw = function() {

        var div = this.div_,
          projection = this.getProjection(),
          ne = projection.fromLatLngToDivPixel( this.marker_.getPosition() ),
          gap = 10,
          posX = ne.x + gap,
          posY = ne.y + gap,
          menuwidth = div.offsetWidth,
          mapNe = this._map.getBounds().getNorthEast(),
          boundsNe = projection.fromLatLngToDivPixel( mapNe );

        if ( menuwidth + posX > boundsNe.x ) {
          posX -= menuwidth + gap;
        }

        div.style.left = posX + 'px';
        div.style.top = posY + 'px';
      };

      Tooltip.prototype.onRemove = function() {
        this.div_.parentNode.removeChild( this.div_ );
      };

      Tooltip.prototype.show = function() {
        var panes;
        if ( this.div_ ) {
          panes = this.getPanes();
          panes.overlayMouseTarget.appendChild( this.div_ );
          this.updateDraw();
        }

      };

      Tooltip.prototype.hide = function() {
        if ( this.div_ ) {
          this.div_.parentNode.removeChild( this.div_ );
        }
      };

      Tooltip.prototype.addClickListener = function( cb ) {
        this.clickListeners.push( cb );
      };
    }

    return Tooltip;
  }
} )( this );
