/**
 * Map Helper
 * @author Yoann
 */

( function( global ) {
  'use strict';

  global.markerVesselTrackHelper = ( global.module || {} ).exports = {
    create: create
  };


  ////////////

  function create( map, positions ) {

    var polylines;

    if ( !positions ) {
      return;
    }

    polylines = _.map( splitPositionByLoading(), function( leg ) {
      return createTrackPolyline( leg.positions, leg.loaded, leg.isCurrentCargo );
    } );

    return {
      polylines: polylines,
      remove: removeTrack,
      setVisibity: setVisibity
    };


    ////////////

    function removeTrack() {
      var i,
        len = polylines.length;

      for ( i = 0; i < len; i++ ) {
        polylines[ i ].setMap( null );
      }

    }

    function setVisibity( visible ) {

      var i,
        len,
        markers = this.markers;

      if ( visible ) {

        len = polylines.length;
        for ( i = 0; i < len; i++ ) {
          polylines[ i ].setMap( map );
        }

        len = markers.length;
        for ( i = 0; i < len; i++ ) {
          if ( !markers[ i ].getMap() ) {
            markers[ i ].setMap( map );
          }
        }

      } else {

        len = polylines.length;
        for ( i = 0; i < len; i++ ) {
          polylines[ i ].setMap( null );
        }

        len = markers.length;
        for ( i = 0; i < len; i++ ) {
          if ( !markers[ i ].getMap() ) {
            markers[ i ].setMap( null );
          }
        }

      }
    }

    function splitPositionByLoading() {
      var positionsLegs = [],
        first = _.first( positions ),
        currentLeg,
        loaded;

      if ( first ) {

        currentLeg = newLeg( positionsLegs, first );

        _( positions )
          .drop()
          .forEach( function( position ) {

            loaded = position.$mapAttrs.$loaded;

            if ( currentLeg.loaded === loaded ) {
              currentLeg.positions.push( position );
            } else {
              currentLeg = newLeg( positionsLegs, position );
            }
          } );
      }

      return positionsLegs;


      ////////////

      function newLeg( legs, position ) {
        var leg = {
          loaded: position.$mapAttrs.$loaded,
          isCurrentCargo: position.$mapAttrs.$isCurrentCargo,
          positions: [ position ]
        };

        legs.push( leg );
        return leg;
      }
    }

    function createTrackPolyline( legPositions, vesselLoaded, isCurrentCargo ) {
      var path,
        loadedColor,
        arrowIcon,
        circleIcon;

      path = _.map( legPositions, function( position ) {
        return position.$mapAttrs.$pos;
      } );

      if ( vesselLoaded ) {
        loadedColor = isCurrentCargo ? '#3B8E5C' : '#757575';
      } else {
        loadedColor = '#f45404';
      }

      arrowIcon = {
        path: google.maps.SymbolPath.BACKWARD_OPEN_ARROW,
        strokeColor: loadedColor,
        scale: 2
      };

      circleIcon = {
        path: google.maps.SymbolPath.CIRCLE,
        strokeColor: '#808080',
        scale: 3
      };

      return new google.maps.Polyline( {
        map: map,
        path: path,
        strokeColor: loadedColor,
        strokeWeight: 4,
        zIndex: google.maps.Marker.MAX_ZINDEX + 200,
        icons: [
          {
            icon: arrowIcon,
            offset: '0',
            repeat: '100px',
            scale: 2
          }, {
            icon: circleIcon,
            offset: '100%'
          }
        ]
      } );
    }
  }
} )( this );
