( function( global ) {
  'use strict';

  global.markerVoyageTrackHelper = ( global.module || {} ).exports = {
    create: create
  };


  ////////////

  function create( map, waypoints ) {
    var path = getTrackPath( waypoints ),
      track = createTrack( map, path );

    return _.partial( removeVoyageTrack, track );

    ////////////

    function removeVoyageTrack( trackToRemove ) {
      trackToRemove.setMap( null );
    }
  }

  function createTrack( map, path ) {
    var primaryColor = '#337ab7',
      arrowIcon,
      circleIcon,
      lineSymbol;

    arrowIcon = {
      path: google.maps.SymbolPath.FORWARD_CLOSED_ARROW,
      strokeColor: primaryColor,
      fillColor: primaryColor,
      fillOpacity: 1,
      strokeOpacity: 1,
      scale: 3
    };

    circleIcon = {
      path: google.maps.SymbolPath.CIRCLE,
      strokeColor: primaryColor,
      strokeOpacity: 1,
      scale: 3
    };

    lineSymbol = {
      path: 'M 0,-1 0,1',
      strokeOpacity: 1,
      scale: 4,
      strokeColor: primaryColor
    };

    return new google.maps.Polyline( {
      map: map,
      path: path,
      zIndex: google.maps.Marker.MAX_ZINDEX + 500,
      strokeOpacity: 0,
      icons: [
        {
          icon: lineSymbol,
          offset: '0',
          repeat: '20px'
        }, {
          icon: arrowIcon,
          offset: '0',
          repeat: '100px',
          scale: 2
        }, {
          icon: circleIcon,
          offset: '100%'
        }
      ]
    } );
  }

  function getTrackPath( waypoints ) {
    return _.map( waypoints, function( position ) {
      return new google.maps.LatLng( position );
    } );
  }

} )( this );
