/**
 * SERVICE: Multicommo
 * A service exposing methods to adapt terms  / display / branding
 * according to the in use commodity
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .factory( 'MulticommoService', multicommoServiceFactory );

  function multicommoServiceFactory( Restangular, SettingsService ) {

    return {
      getAvailableUnits: getAvailableUnits,
      getCurrentCommodity: getCurrentCommodity
    };

    //////////////////

    /**
     * Get the commodity the user account is linked to
     * Useful to adapt branding elements of the UI (page title, logo ...)
     * @returns {object} commodity tied to current user account (one user account <=> one commodity)
     */
    function getCurrentCommodity() {
      var settings = SettingsService.getSettings();

      switch ( settings.COMMODITY ) {
        case 'LNG':
          return {
            name: 'lng',
            id: 1
          };
        case 'LPG':
          return {
            name: 'lpg',
            id: 2
          };
        case 'OIL':
          return {
            name: 'oil',
            id: 8
          };
      }
    }

    /**
     * GET from server side the collection of available units for a specific commodity (the one linked to the current account)
     * @returns {promise}
     */
    function getAvailableUnits() {
      var currentCommodity = getCurrentCommodity();

      return Restangular.one( 'commodities', currentCommodity.id )
        .getList( 'units' )
        .then( function( data ) {
          return data.plain();
        } );

    }
  }

} )();
