/**
 * DIRECTIVE: portcallKnown
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .directive( 'portcallKnown', portcallKnown );

  function portcallKnown() {
    var directive = {
      restrict: 'E',
      scope: {},
      bindToController: {
        portcall: '=',
        type: '='
      },
      templateUrl: 'components/portcall/known/portcall-known.html',
      controller: controllerFunc,
      controllerAs: 'vm'
    };

    return directive;


    ////////////

    /* @ngInject */
    function controllerFunc(
      LoginService,
      MapCacheService,
      MulticommoService,
      PhysicalQuantityService
    ) {

      var vm = this;
      vm.maxCapacity = _.flowRight( PhysicalQuantityService.getValue, MapCacheService.getMaxCapacity )( vm.portcall.vessel );

      updatePortcall( this.portcall );


      ////////////

      function updatePortcall( portcall ) {
        var isTracked = global.displayHelper.isCurrentlyTracked( global.displayHelper.getFlattenedAncestorIdsFromVoyageProducts( portcall.voyageProducts || portcall.flowQuantities ) );
        vm.$showCargoVolume = isTracked;
        vm.portcall = portcall;
        vm.portcall.$cargoQuantity = PhysicalQuantityService.getValue( vm.portcall.flowQuantity );
        vm.clazz = getClazz( isTracked );

        ////////////

        function getClazz( isTracked ) {

          var clazz = '';

          // This PC is not for the current commo, grey it out.
          if ( !isTracked ) {
            clazz += 'other-commo';
          }

          if ( portcall.forecasted || portcall.future ) { clazz += ' future'; }
          if ( ( portcall.$start && !portcall.$end ) || ( portcall.arrival && !portcall.departure ) ) { clazz += ' at-berth'; }
          if ( portcall.installation && portcall.installation.type === 'Anchorage' ) { clazz += ' anchorage'; }
          if ( portcall.cancelled ) { clazz += ' cancelled'; }
          if ( portcall.isGhost ) { clazz = 'ghost-portcall'; }

          return clazz;
        }
      }
    }
  }

} )( this );
