/**
 * DIRECTIVE: searchBox
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .directive( 'searchBox', searchBox );

  function searchBox( $timeout ) {
    var directive = {
      restrict: 'E',
      scope: {},
      templateUrl: 'components/search-box/search-box.html',
      controller: controllerFunc,
      controllerAs: 'vm'
    };

    return directive;


    ////////////

    /* @ngInject */
    function controllerFunc( $scope, $state ) {
      var vm = this,
        searchEl = document.getElementById( 'search' ),
        timer,
        stateChangeSuccess;

      vm.request = '';
      vm.enter = enter;
      vm.clean = clean;
      vm.registerRequest = registerRequest;

      stateChangeSuccess = $scope.$on( '$stateChangeSuccess', function( e, toState, toParams, fromState ) {

        if ( toState.name === 'session.search' ) {
          if ( fromState.name === 'session.search' ) {
            vm.request = toParams.searchRequest;
          }
          timer = $timeout( searchEl.focus );

        } else {
          clean();
        }

      } );

      $scope.$on( '$destroy', function() {
        stateChangeSuccess();
        $timeout.cancel( timer );
      } );


      function enter() {
        if ( $state.current.name !== 'session.search' ) {
          $state.go( 'session.search' );
          timer = $timeout( searchEl.focus );
        }
      }

      function registerRequest() {
        $scope.$emit( 'search:change', vm.request );
      }

      function clean() {
        vm.request = '';
        $scope.$emit( 'search:change', vm.request );
      }

    }
  }
} )();
