/**
 * DIRECTIVE: teamboardCompile
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function() {
  'use strict';


  angular
    .module( 'lng' )
    .directive( 'teamboardCompile', teamboardCompile );

  function teamboardCompile( $compile ) {
    var directive = {
      restrict: 'A',
      scope: {
        content: '='
      },
      link: linkFunc
    };

    return directive;


    ////////////

    function linkFunc( scope, element ) {
      var unwatchContent = scope.$watch( 'content', function( newvalue ) {
        if ( newvalue ) {
          updateContent( newvalue );
          unwatchContent();
        }
      } );


      ////////////

      function updateContent( content ) {
        var result = content.replace( /#(.*?)#/g, stringToObject );
        element.html( result );
        $compile( element.contents() )( scope );
      }

      function stringToObject( match, p1 ) {
        var json;
        try {
          json = JSON.parse( p1 );
          return objectToHtml( json );
        } catch ( error ) {
          return '';
        }
      }

      function objectToHtml( obj ) {

        var state,
          idKey,
          // Features not yet included in the mobile app
          blacklisted = [ 'zone', 'player' ];

        if ( !obj.resourceType || !obj.id || ( blacklisted.indexOf( obj.resourceType ) > -1 ) ) {
          return obj.name;
        }

        if ( obj.resourceType === 'vessel' ) {
          state = 'session.map.vessel';
          idKey = 'id';
        }

        if ( obj.resourceType === 'installation' ) {
          state = 'session.installation';
          idKey = 'itemId';
        }

        return '<a ui-sref="' + state + '({' + idKey + ':' + obj.id + ', zoom: true})" ui-sref-opts="{reload:true}">' + obj.name + '</a>';
      }
    }
  }

} )();
