/**
 * Dates Helper
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';

  global.datesUtils = ( global.module || {} ).exports = {
    momentFullDate: momentFullDate,
    momentShortHumanDate: momentShortHumanDate,
    humanize: humanize,
    momentFullTime: momentFullTime
  };


  ////////////

  // DD-MM-YYYY
  function momentFullDate( mdate ) {
    return momentDate( mdate, 'DD-MM-YYYY' );
  }

  function momentFullTime( mdate ) {
    return momentDate( mdate, 'YYYY-MM-DD HH:mm' );
  }

  // D MMM
  function momentShortHumanDate( mdate ) {
    return momentDate( mdate, 'D-MMM' );
  }

  function humanize( mDate ) {
    var d;
    if ( mDate ) {
      //make a moment object if not already, but don'Mt with null
      d = !moment.isMoment( mDate ) ? moment.utc( mDate ) : mDate;

      if ( d.isValid() ) {
        // short output text
        return shortFromNow( d.fromNow() );
      } else {
        return ' - ';
      }
    }

    return ' - ';
  }

  function momentDate( mdate, format ) {
    if ( mdate && ( typeof mdate.format !== 'function' ) ) {
      mdate = moment.utc( mdate );
    }

    return mdate ? mdate.format( format ) : ' - ';
  }

  function shortFromNow( moment ) {

    if ( moment.indexOf( 'minute' ) > 0 ) {
      return moment.replace( 'minute', 'mn' );
    }

    if ( moment.indexOf( 'second' ) > 0 ) {
      return moment.replace( 'second', 'sec' );
    }

    return moment;
  }

} )( this );
