( function() {

  'use strict';

  angular
    .module( 'lng' )
    .directive( 'typeahead', typeaheadDirective );

  function typeaheadDirective( $compile, $timeout ) {

    var directive = {
      restrict: 'A',
      require: 'ngModel',
      scope: {
        renderOption: '&',
        findOptions: '&',
        showValue: '='
      },
      link: linkFunc
    };

    return directive;

    ////////////

    function linkFunc( scope, element, attrs, ngModelController ) {
      var autocompletionList;

      element.bind( 'focus', toggleList( true ) );
      element.bind( 'blur', toggleList( false ) );

      scope.getOptionValue = getOptionValue;
      scope.selectItem = selectItem;

      element.val( _.get( scope, 'showValue' ) );

      ngModelController.$render = renderInputValue;
      ngModelController.$parsers.push( parseViewValue );
      ngModelController.$validators.parse = function( mv ) { return _.isObject( mv ); };

      autocompletionList = $compile( '<ion-list ng-show="showValues"><ion-item class="item-icon-left" ng-repeat="item in options" ng-click="selectItem(item)" ng-bind-html="getOptionValue(item)"></ion-item></ion-list>' )( scope );
      element.after( autocompletionList );

      ////////////

      function toggleList( show ) {
        return function() { scope.$evalAsync( function() { scope.showValues = show; } ); };
      }

      function getOptionValue( item ) {
        if ( scope.renderOption ) { return scope.renderOption( { $item: item } ); }
        return item;
      }

      function selectItem( item ) {
        // Timeout required to apply the selection properly
        // Otherwise, ionNgClick will trigger another digest
        // and will update ngModelController.viewValue to be a string.
        // which makes the form to be invalid.
        $timeout( function() {
          // First assign the selected item (object) to ngModel
          ngModelController.$setViewValue( item );
          // Then render it so that we display only its name
          ngModelController.$render();
          scope.showValues = false;
        } );
      }

      /**
       * This could be generalized
       * by moving to the parent contect the definition of the function
       * defining how to render the viewValue.
       * This was the case before, but as this directive has only one use right now,
       * simpler to not generalize it right now.
       * */
      function renderInputValue() {
        var renderedValue = _.get( scope, 'showValue' );
        element.val( renderedValue );
      }

      function parseViewValue( viewValue ) {
        var promise;
        if ( _.isObject( viewValue ) ) {
          return viewValue;
        } else {
          promise = scope.findOptions( { query: viewValue } );
          if ( promise ) {
            promise.then( function( options ) {
              scope.options = ( options ? options.splice( 0, 10 ) : null );
            } );
          }
        }
      }
    }
  }

} )();
