/**
 * DIRECTIVE: tradeLink
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

( function( global ) {
  'use strict';


  angular
    .module( 'lng' )
    .directive( 'tradeLink', tradeLink );

  function tradeLink() {
    var directive = {
      restrict: 'E',
      scope: {},
      bindToController: {
        tradeLink: '='
      },
      templateUrl: 'components/trades/trade/trade-link/trade-link.html',
      controller: TradeLinkController,
      controllerAs: 'vm'
    };

    return directive;


    ////////////

    /* @ngInject */
    function TradeLinkController( SettingsService ) {
      var vm = this;

      vm.showPotentialPlayers = showPotentialPlayers;
      vm.showPotentialSellers = false;
      vm.showPotentialBuyers = false;
      vm.isOil = SettingsService.getSettings().COMMODITY;

      processTradeLink();

      ////////////

      /**
       * Process the tradelink object to add / create info needed by the frontend
       *   - Sellers / Buyers object
       *   - A string listing potential sellers / potential buyers
       *   - A class specific to the type of the tradelink
       * @param {object} tradeLink object
       */
      function processTradeLink() {

        // get the array of sellers without duplicates
        vm.sellers = _.values(
          vm.tradeLink.seller.reduce( function( object, seller ) {
            object[ seller.id ] = seller;
            return object;
          }, {} )
        );

        // get the array of buyers without duplicates
        vm.buyers = _.values(
          vm.tradeLink.buyer.reduce( function( object, buyer ) {
            object[ buyer.id ] = buyer;
            return object;
          }, {} )
        );

        //The following two computed strings will be used in directional-bar tooltips where HTML
        //can be used
        vm.sellersListString = vm.sellers.map( function( seller ) {
          return seller.name;
        } ).join( ', ' );

        vm.buyersListString = vm.buyers.map( function( buyer ) {
          return buyer.name;
        } ).join( ', ' );

        // Add class specific to the tradelink
        vm.tradeLinkTypeClazz = function( type ) {
          return global.clazzHelper.getTypeClazz( type );
        };
      }

      /**
       * Show hide List of potential buyers / sellers
       * @param {string} type : 'sellers' or 'buyers' or 'none' (default)
       */
      function showPotentialPlayers( type ) {
        if ( type === 'sellers' ) {
          vm.showPotentialBuyers = false;
          vm.showPotentialSellers = !vm.showPotentialSellers;
        } else if ( type === 'buyers' ) {
          vm.showPotentialSellers = false;
          vm.showPotentialBuyers = !vm.showPotentialBuyers;
        } else {
          vm.showPotentialSellers = false;
          vm.showPotentialBuyers = false;
        }
      }

    }
  }

} )( this );
