/**
 * Application configuration
 * @author Fabien Vauchelles <fabien@vauchelles.com>
 */

/* exported handleOpenURL */

( function() {
  'use strict';


  angular.module( 'lng', [
    'ionic',
    'ngCordova',
    'restangular',
    'rzModule',
    'pascalprecht.translate'
  ] )
    .run( AppConfig );

  function AppConfig(
    $ionicPlatform,
    $state,
    $timeout,
    $window,
    NetworkInformationService
  ) {

    $ionicPlatform.ready( function() {

      var resumedAt,
        pausedAT,
        delayBeforeRestart = 15 * 60 * 1000;

      // org.apache.cordova.statusbar required
      if ( window.StatusBar ) { window.StatusBar.styleDefault(); }

      document.addEventListener( 'deviceready', function() {
        console.info( 'App started. Device ready.' );
        ionic.Platform.isFullScreen = false;
      }, false );


      // Handling network connexion state
      document.addEventListener( 'online', NetworkInformationService.hasBeenReconnected, false );
      document.addEventListener( 'offline', NetworkInformationService.hasBeenDisconnected, false );

      // Handling app when switching between foreground / background modes
      document.addEventListener( 'pause', function() { pausedAT = Date.now(); }, false );
      document.addEventListener( 'resume', function() {
        // App switches to foreground mode
        resumedAt = Date.now();
        // Restart the app is it stayed too long on the background
        // Configure delayBeforeRestart variable
        if ( ( resumedAt - pausedAT ) > delayBeforeRestart ) {
          $timeout( function() {
            $state.go( 'start', {}, { reload: true } );
          } );
        }
      }, false );

    } );
  }
} )();

////////////

/**
 * Custom URL scheme PhoneGap Plugin
 * sources:
 *  - https://github.com/EddyVerbruggen/Custom-URL-scheme#3-usage
 *  - http://forum.ionicframework.com/t/ios-url-schema-handling/3998/8
 */
function handleOpenURL( url ) {

  'use strict';

  url = url.replace( 'kpler://', '' );
  // We store the url to open to be able to make the redirection once the app is initialized
  // @see: init.controller.js file
  if ( url.length ) { localStorage.setItem( 'urlFromScheme', url ); }

}
